import Flutter
import UIKit

public class SwiftBlotoutfluttersdkPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "blotoutfluttersdk", binaryMessenger: registrar.messenger())
    let instance = SwiftBlotoutfluttersdkPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    
    switch call.method {
    case Initialize_SDK:
      self.initializeAnalyticsEngineUsingTest(call: call, result: result);
      break;
    case Update_Blotout_SDK:
      self.updateAnalyticsEngineTest(call: call, result:result);
     break;
    case Start_Timed_Event:
    self.startTimedEvent(call: call, result:result);
     break;
      case End_Timed_Event:
      self.endTimedEvent(call: call, result:result);
     break;
      case Log_Event:
      self.logEvent(call: call, result:result);
     break;
      case Log_Event_With_Time:
      self.logEventWithTime(call: call, result:result);
     break;
      case Log_PII_Event:
      self.logPIIEvent(call: call, result:result);
     break;
      case Log_PHI_Event:
      self.logPHIEvent(call: call, result:result);
     break;
       case Set_SDK_End_Point_Url:
       self.setSDKEndPointUrl(call: call, result:result);
     break;
       case Set_SDK_Enabled:
       self.setSDKEnabled(call: call, result:result);
     break;

    default:
      break;
    }
  }
}


extension SwiftBlotoutfluttersdkPlugin {
 
    let Initialize_SDK = "initBlotoutSDK"
    let Update_Blotout_SDK = "updateBlotoutSDK"
    let Start_Timed_Event = "startTimedEvent"
    let End_Timed_Event = "endTimedEvent"
    let Start_Timed_Event = "startTimedEvent"
    let Log_Event = "logEvent";
    let Log_Event_With_Time = "logEventWithTime";
    let Log_User_Retention_Event = "logUserRetentionEvent";
    let Log_PII_Event = "logPIIEvent";
    let Log_PHI_Event = "logPHIEvent";
   
    let Set_SDK_End_Point_Url = "setSDKEndPointUrl";
    let Set_SDK_Enabled = "setSDKEnabled";
    let Set_Network_Sync_Enabled = "setNetworkSyncEnabled";
    let Set_Data_Collection_Enabled = "setDataCollectionEnabled";
    let Set_Funnel_Events_Enabled = "setFunnelEventsEnabled";
    let Set_Segment_Events_Enabled = "setSegmentEventsEnabled";
    let Set_Retention_Events_Enabled = "setRetentionEventsEnabled";
    let Set_System_Events_Enabled = "setSystemEventsEnabled";
    let Set_Developer_Events_Enabled = "setDeveloperEventsEnabled";

    let IS_Device_Compromised = "isDeviceCompromised";
    let IS_App_Compromised = "isAppCompromised";
    let IS_Network_Proxied = "isNetworkProxied";
    let IS_Simulator = "isSimulator";
    let IS_Running_On_VM = "isRunningOnVM";
    let IS_Environment_Secure = "isEnvironmentSecure";
}

extension SwiftBlotoutfluttersdkPlugin {

  private func initializeAnalyticsEngineUsingTest(call: FlutterMethodCall, result: @escaping FlutterResult){

  }


/*
 * @param blotoutTestKey stage key as String
 * @param blotoutProductionKey production key as String
 */
private func updateAnalyticsEngineTest(call: FlutterMethodCall, result: @escaping FlutterResult) {
  try {
    [[BlotoutAnalytics sharedInstance] updateAnalyticsEngineTest:blotoutTestKey andProduction:blotoutProductionKey];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/*
 * @param eventName name of the event
 * @param startEventInfo properties in key/value pair
 */
private func startTimedEvent(call: FlutterMethodCall, result: @escaping FlutterResult) {
  try {
    [[BlotoutAnalytics sharedInstance] startTimedEvent:eventName withInformation:startEventInfo];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/*
 * @param eventName name of the event
 * @param endEventInfo properties in key/value pair
 */
private func endTimedEvent(call: FlutterMethodCall, result: @escaping FlutterResult){
  try {
    [[BlotoutAnalytics sharedInstance] endTimedEvent:eventName withInformation:endEventInfo];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/*
 * @param eventName name of the event
 * @param eventInfo properties in key/value pair
 */
private func logEvent(call: FlutterMethodCall, result: @escaping FlutterResult) {
  try {
    [[BlotoutAnalytics sharedInstance] logEvent:eventName withInformation:eventInfo];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/*
 * @param eventName name of the event as String
 * @param eventInfo properties in key/value pair
 * @param eventTime eventTime as Date
 */
private func logEventWithTime(call: FlutterMethodCall, result: @escaping FlutterResult){
  try {
    NSDate *date = [RCTConvert NSDate:eventTime];
    [[BlotoutAnalytics sharedInstance] logEvent:eventName withInformation:eventInfo happendAt:date];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/*
 * @param eventName name of the event as String
 * @param eventInfo properties in key/value pair
 */
private func logUserRetentionEvent(call: FlutterMethodCall, result: @escaping FlutterResult){
  @try {
    [[BlotoutAnalytics sharedInstance] logUserRetentionEvent:eventName withInformation:eventInfo];
  } @catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/*
 * @param eventName name of the event as String
 * @param eventInfo properties in key/value pair
 */
private func logPIIEvent(call: FlutterMethodCall, result: @escaping FlutterResult){
  try{
    NSDate *date = [RCTConvert NSDate:eventTime];
    [[BlotoutAnalytics sharedInstance] logPIIEvent:eventName withInformation:eventInfo happendAt:date];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/*
 * @param eventName name of the event as String
 * @param eventInfo properties in key/value pair
 */
private func logPHIEvent(call: FlutterMethodCall, result: @escaping FlutterResult){
  try{
    NSDate *date = [RCTConvert NSDate:eventTime];
    [[BlotoutAnalytics sharedInstance] logPHIEvent:eventName withInformation:eventInfo happendAt:date];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

/**
 * SDK Endpoint Url must be set before Analytics Engine Initialization, It has to be a domain based IP's are considered invalid.
 * @param endPointUrl 
 */


private func call: FlutterMethodCall, result: @escaping FlutterResult) {
  try{
    [[BlotoutAnalytics sharedInstance] setSDKEndPointUrl:endPointUrl];
  } catch (NSException *exception) {
    NSLog(@"%@",[exception description]);
  }
}

}