import 'dart:async';
import 'package:flutter/services.dart';

class BlotoutAnalyticsAPI {
  static const String _pluginName = 'blotoutfluttersdk';

  static const MethodChannel _channel = const MethodChannel(_pluginName);

  BlotoutAnalyticsAPI._privateConstructor();

  static final BlotoutAnalyticsAPI blotoutAnalyticsInstance =
      BlotoutAnalyticsAPI._privateConstructor();

  factory BlotoutAnalyticsAPI() {
    return blotoutAnalyticsInstance;
  }

  static Future<bool> initBlotoutSDK(
      String testKey, String prodKey, bool isProductionMode) async {
    Map<String, dynamic> properties = <String, dynamic>{
      'testKey': testKey,
      'prodKey': prodKey,
      'isProdMode': isProductionMode
    };

    bool sdkInitialized =
        await _channel.invokeMethod<bool>('initBlotoutSDK', properties);
    return sdkInitialized;
  }

  Future<bool> updateBlotoutSDK(String testKey, String prodKey) async {
    Map<String, dynamic> properties = <String, dynamic>{
      'testKey': testKey,
      'prodKey': prodKey
    };

    bool sdkUpdated =
        await _channel.invokeMethod<bool>('updateBlotoutSDK', properties);
    return sdkUpdated;
  }

  void setSDKEndPointUrl(String endPointUrl) {
    Map<String, dynamic> properties = <String, dynamic>{
      'endPoint': endPointUrl
    };

    _channel.invokeMethod<void>('setSDKEndPointUrl', properties);
  }

  void setSDKEnabled(bool sdkEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'sdkEnabled': sdkEnabled
    };

    _channel.invokeMethod<void>('setSDKEnabled', eventData);
  }

  void logEvent(String eventName, Map<String, dynamic> properties) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'eventName': eventName,
      'properties': properties
    };

    _channel.invokeMethod<void>('logEvent', eventData);
  }

  void logEventWithTime(
      String eventName, Map<String, dynamic> properties, DateTime time) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'eventName': eventName,
      'properties': properties,
      'time': time
    };

    _channel.invokeMethod<void>('logEventWithTime', eventData);
  }

  void startTimedEvent(String eventName, Map<String, dynamic> properties) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'eventName': eventName,
      'properties': properties
    };

    _channel.invokeMethod<void>('startTimedEvent', eventData);
  }

  void endTimedEvent(String eventName, Map<String, dynamic> properties) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'eventName': eventName,
      'properties': properties
    };

    _channel.invokeMethod<void>('endTimedEvent', eventData);
  }

  void logUserRetentionEvent(
      String eventName, Map<String, dynamic> properties) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'eventName': eventName,
      'properties': properties
    };

    _channel.invokeMethod<void>('logUserRetentionEvent', eventData);
  }

  void logPIIEvent(String eventName, Map<String, dynamic> properties) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'eventName': eventName,
      'properties': properties
    };

    _channel.invokeMethod<void>('logPIIEvent', eventData);
  }

  void logPHIEvent(String eventName, Map<String, dynamic> properties) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'eventName': eventName,
      'properties': properties
    };

    _channel.invokeMethod<void>('logPHIEvent', eventData);
  }

  /*
 * @param networkSyncEnabled default is true, set false to disable the Network call
 */

  void setNetworkSyncEnabled(bool networkSyncEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'enabled': networkSyncEnabled
    };

    _channel.invokeMethod<void>('setNetworkSyncEnabled', eventData);
  }

/*
 * @param dataCollectionEnabled default is true, set false to disable data collection
 */

  void setDataCollectionEnabled(bool dataCollectionEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'enabled': dataCollectionEnabled
    };

    _channel.invokeMethod<void>('setDataCollectionEnabled', eventData);
  }

/*
 *
 * @param funnelEventsEnabled default is true, set false to disable funnel execution
 */

  void setFunnelEventsEnabled(bool funnelEventsEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'enabled': funnelEventsEnabled
    };

    _channel.invokeMethod<void>('setFunnelEventsEnabled', eventData);
  }

/*
 *
 * @param segmentEventsEnabled default is true, set false to disable segment execution
 */

  void setSegmentEventsEnabled(bool segmentEventsEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'enabled': segmentEventsEnabled
    };

    _channel.invokeMethod<void>('setSegmentEventsEnabled', eventData);
  }

/*
 * @param retentionEventsEnabled default is true, set false to disable retention events execution
 */

  void setRetentionEventsEnabled(bool retentionEventsEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'enabled': retentionEventsEnabled
    };

    _channel.invokeMethod<void>('setRetentionEventsEnabled', eventData);
  }

/*
 *
 * @param developerEventsEnabled default is true, set false to disable developer events execution
 */

  void setDeveloperEventsEnabled(bool developerEventsEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'enabled': developerEventsEnabled
    };

    _channel.invokeMethod<void>('setDeveloperEventsEnabled', eventData);
  }

/*
 *
 * @param systemEventsEnabled default is true, set false to disable device/system events execution
 */

  void setSystemEventsEnabled(bool systemEventsEnabled) {
    Map<String, dynamic> eventData = <String, dynamic>{
      'enabled': systemEventsEnabled
    };

    _channel.invokeMethod<void>('setSystemEventsEnabled', eventData);
  }

/*
 *
 * @param payingUser default is false, set true to enable the payingUser analytics
 */

  void setPayingUser(bool payingUser) {
    Map<String, dynamic> eventData = <String, dynamic>{'enabled': payingUser};

    _channel.invokeMethod<void>('setPayingUser', eventData);
  }

  //Fraud Services
  Future<bool> isDeviceCompromised() async {
    bool value = await _channel.invokeMethod<bool>('isDeviceCompromised');
    return value;
  }

  //Fraud Services
  Future<bool> isAppCompromised() async {
    bool value = await _channel.invokeMethod<bool>('isAppCompromised');
    return value;
  }

//Fraud Services
  Future<bool> isNetworkProxied() async {
    bool value = await _channel.invokeMethod<bool>('isNetworkProxied');
    return value;
  }

//Fraud Services
  Future<bool> isSimulator() async {
    bool value = await _channel.invokeMethod<bool>('isSimulator');
    return value;
  }

//Fraud Services
  Future<bool> isRunningOnVM() async {
    bool value = await _channel.invokeMethod<bool>('isRunningOnVM');
    return value;
  }

//Fraud Services
  Future<bool> isEnvironmentSecure() async {
    bool value = await _channel.invokeMethod<bool>('isEnvironmentSecure');
    return value;
  }
}
